// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef LIBKOMMIT_VERSION_H
#define LIBKOMMIT_VERSION_H

#define LIBKOMMIT_VERSION_STRING "1.0.0"
#define LIBKOMMIT_VERSION_MAJOR 1
#define LIBKOMMIT_VERSION_MINOR 0
#define LIBKOMMIT_VERSION_PATCH 0
#define LIBKOMMIT_VERSION ((1<<16)|(0<<8)|(0))

#endif
